﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;

namespace imaeg {
    class ImgMan {
        const int MAXQUE = 5;

        string root;
        List<Img> image;
        List<Img> queue;
        public ImgFilter filter;
        public ImgMan(string root, string[] names) {
            //debug();
            this.root = root;
            image = new List<Img>();
            queue = new List<Img>();
            foreach (string name in names)
                image.Add(new Img(name));

            filter = new ImgFilter();
            Thread loada = new Thread(new ThreadStart(loader));
            loada.Priority = ThreadPriority.BelowNormal;
            loada.IsBackground = true;
            loada.Start();
        }



        Label lb;
        void debug() {
            lb = new Label();
            lb.Visible = true;
            lb.Dock = DockStyle.Fill;
            Form fm = new Form();
            fm.Controls.Add(lb);
            fm.Size = new Size(320, 900);
            fm.TopMost = true;
            fm.Show();
            System.Windows.Forms.Timer t = new System.Windows.Forms.Timer();
            t.Tick += new EventHandler(t_Tick);
            t.Interval = 100;
            t.Start();
        }
        void t_Tick(object sender, EventArgs e) {
            StringBuilder a = new StringBuilder();
            StringBuilder b = new StringBuilder();
            for (int i = 0; i < image.Count; i++) {
                if (image[i].loaded) {
                    a.AppendLine(i + " - " + image[i].getName());
                }
            }
            for (int i = 0; i < queue.Count; i++) {
                b.AppendLine(i + " - " + queue[i].getName());
            }
            lb.Text = System.DateTime.Now.Ticks +
                "\n\n" + a.ToString() +
                "\n\n" + b.ToString();
        }



        private Image getRaw(int i) {

            // Start off with unloading excessive images
            for (int a = 0; a < i - 3; a++) {
                image[a].dispose();
            }
            for (int a = i + 3; a < image.Count; a++) {
                image[a].dispose();
            }

            // New filter settings?
            if (filter.hasChanged()) { //ALTERNATIVE_CODE
                foreach (Img im in image) {
                    im.dispose();
                }
            }

            queue.Clear();
            queue.Add(image[i]);
            enqueue(i + 1);
            enqueue(i + 2);
            enqueue(i - 1);
            enqueue(i - 2);

            if (image[i].loaded) {
                // Already loaded
                return image[i].get();

            } else {

                /* Remove excessive items
                if (queue.Count > MAXQUE) {
                    queue.RemoveRange(MAXQUE,
                        queue.Count - MAXQUE);
                }*/

                // Wait for it...
                while (!image[i].loaded)
                    Thread.Sleep(1);
                return image[i].get();
            }
        }

        /// <summary>
        /// Enqueue image i at a lower priority (caching)
        /// </summary>
        /// <param name="i"></param>
        void enqueue(int i) {
            if (i < 0 || i >= image.Count) return;
            if (!image[i].loaded) {
                //foreach (Img j in queue)
                    //if (image[i] == j) return;
                queue.Add(image[i]);
                //queue.Insert(1, image[i]);
            }
        }

        /// <summary>
        /// Worker thread; loads images.
        /// </summary>
        void loader() {
            while (true) {
                if (queue.Count > 0) {
                    Img i = queue[0];
                    queue.RemoveAt(0);
                    if (!i.loaded) {
                        //i.load(root);
                        i.load(root, filter); //ALTERNATIVE_CODE
                        //i.set(filter.apply(i.get())); //ALTERNATIVE_CODE
                    }
                } else {
                    Thread.Sleep(10);
                }
            }
        }

        /// <summary>
        /// Returns image i, applying all configured enhancements
        /// </summary>
        /// <param name="i">The image to return</param>
        /// <returns>Image i</returns>
        public Image get(int i) {
            //return filter.apply(getRaw(i));
            return (Image)getRaw(i).Clone(); //ALTERNATIVE_CODE
        }

        public int length {
            get { return image.Count; }
            set { }
        }
        public string getName(int i) {
            return image[i].getName();
        }
        public void setName(int i, string s) {
            image[i].setName(s);
        }
        public void remove(int i) {
            image.RemoveAt(i);
        }
    }
}
